using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.IO;
using System.Linq;
using System.Text.Json;
using JetBrains.Annotations;
using PkmnLib.Dataloader.Models;
using PkmnLib.Static;
using PkmnLib.Static.Libraries;
using PkmnLib.Static.Moves;
using PkmnLib.Static.Utils;

namespace PkmnLib.Dataloader;

public static class ItemDataLoader
{
    public static ItemLibrary LoadItems(Stream stream)
    {
        var library = new ItemLibrary();
        var obj = JsonSerializer.Deserialize<SerializedItem[]>(stream, JsonOptions.DefaultOptions);
        if (obj == null)
            throw new InvalidDataException("Item data is empty.");
        var items = obj.Select(DeserializeItem);
        foreach (var i in items)
            library.Add(i);
        return library;
    }

    public delegate IItem ItemFactoryDelegate(SerializedItem serialized, StringKey name, ItemCategory type,
        BattleItemCategory battleType, int price, ImmutableHashSet<StringKey> flags, ISecondaryEffect? effect,
        ISecondaryEffect? battleTriggerEffect, byte flingPower);

    [PublicAPI]
    public static ItemFactoryDelegate ItemConstructor { get; set; } =
        (_, name, type, battleType, price, flags, effect, battleTriggerEffect, flingPower) => new ItemImpl(name, type,
            battleType, price, flags, effect, battleTriggerEffect, flingPower);

    private static IItem DeserializeItem(SerializedItem serialized)
    {
        if (!Enum.TryParse<ItemCategory>(serialized.ItemType, true, out var itemType))
            throw new InvalidDataException($"Item type {serialized.ItemType} is not valid for item {serialized.Name}.");
        Enum.TryParse(serialized.BattleType, true, out BattleItemCategory battleType);
        var effect = serialized.Effect?.ParseEffect();
        var battleTriggerEffect = serialized.BattleEffect?.ParseEffect();

        return ItemConstructor(serialized, serialized.Name, itemType, battleType, serialized.Price,
            serialized.Flags.Select(x => (StringKey)x).ToImmutableHashSet(), effect, battleTriggerEffect,
            serialized.FlingPower);
    }
}