using PkmnLib.Dynamic.ScriptHandling;

namespace PkmnLib.Dynamic.Models.Choices;

/// <summary>
/// A choice to switch to a different Pokémon.
/// </summary>
public interface ISwitchChoice : ITurnChoice
{
    /// <summary>
    /// The Pokémon to switch to.
    /// </summary>
    IPokemon SwitchTo { get; }
}

/// <inheritdoc cref="ISwitchChoice"/>
public class SwitchChoice : TurnChoice, ISwitchChoice
{
    public SwitchChoice(IPokemon user, IPokemon switchTo) : base(user)
    {
        SwitchTo = switchTo;
    }

    /// <inheritdoc />
    public IPokemon SwitchTo { get; }

    /// <inheritdoc />
    public override int ScriptCount => User.ScriptCount;

    /// <inheritdoc />
    public override void GetOwnScripts(List<IEnumerable<ScriptContainer>> scripts)
    {
    }

    /// <inheritdoc />
    public override void CollectScripts(List<IEnumerable<ScriptContainer>> scripts)
    {
        User.CollectScripts(scripts);
    }
}